/*
	pmserv.rc
*/

#include <os2.h>
#include "pmdefs.h"

MENU ID_RESOURCE
    {
    MENUITEM "~Advance", IDM_ADVANCE

    SUBMENU "~Controls", IDM_CONTROLS
	{
	    MENUITEM "~Auto Advance", IDM_AUTO_ADVANCE
	}
    }
