# $Id: demos.tcl,v 1.3 1994/06/30 18:47:34 mjl Exp $
# $Log: demos.tcl,v $
# Revision 1.3  1994/06/30  18:47:34  mjl
# Now sources all the tcl demo files.
#
# Revision 1.2  1994/06/25  20:33:54  mjl
# Moved stuff corresponding to first demo into separate file.
#
# Revision 1.1  1994/06/23  22:39:30  mjl
# New demo file for "pltcl" interpreter.  Needs fleshing out.
#
#----------------------------------------------------------------------------
# PLPLOT TCL demos
# 
# Maurice LeBrun
# IFS, University of Texas at Austin
# 23-Jun-1994
#
# To plot these, start up pltcl and select the desired device type.  Type
# "source demos.tcl", then type "1" for the first demo, "2" for the
# second, and so on.
#----------------------------------------------------------------------------

source x01.tcl
source x02.tcl
source x03.tcl
source x04.tcl
source x05.tcl
source x06.tcl
source x07.tcl
source x08.tcl
source x09.tcl
source x10.tcl
source x11.tcl
source x12.tcl
source x13.tcl
source x14.tcl
source x15.tcl
source x16.tcl
