;	pmserv.def
;	Geoffrey Furnish
;	10-16-91
;
;	.def file for pmserv.c, the graphics output server for PLPLOT
;	under OS/2 Presentation Manager.

name	pmserv	windowapi

description	'PLPLOT OS/2 PM Graphics Server.'

protmode
heapsize	1024
stacksize	8192
exports		ClientWndProc
