int plamiga_Open(void);
int plamiga_Save(void);
int plamiga_SaveAs_ILBM(void);
int plamiga_Print_Bitmap(void);
int plamiga_Print_landscape(void);
int plamiga_Print_portrait(void);
int plamiga_About(void);
int plamiga_Quit(void);
int plamiga_Screenmode(void);
int plamiga_Palette0(void);
int plamiga_Palette1(void);
int plamiga_LoadConfig(void);
int plamiga_SaveConfigAs(void);
int plamiga_SaveConfig(void);
int plamiga_DUMMY(void);
int plamiga_KEY(void);
